package com.kernelmaft.zanbur

import kotlinx.serialization.encodeToString
import kotlinx.serialization.json.Json
import android.os.Bundle
import androidx.activity.ComponentActivity
import androidx.activity.compose.setContent
import androidx.activity.enableEdgeToEdge
import androidx.compose.foundation.layout.Arrangement.Center
import androidx.compose.foundation.layout.Column
import androidx.compose.foundation.layout.fillMaxHeight
import androidx.compose.foundation.layout.fillMaxWidth
import androidx.compose.foundation.layout.padding
import androidx.compose.material3.Button
import androidx.compose.material3.ButtonDefaults
import androidx.compose.material3.Scaffold
import androidx.compose.material3.Text
import androidx.compose.runtime.Composable
import androidx.compose.runtime.getValue
import androidx.compose.runtime.setValue
import androidx.compose.ui.Alignment.Companion.CenterHorizontally
import androidx.compose.ui.Modifier
import androidx.lifecycle.lifecycleScope


class MainActivity : ComponentActivity () {
	override fun onCreate ( savedInstanceState : Bundle ? ) {
		super . onCreate (savedInstanceState)

		val devices by AppState . devices
		val scenes by AppState . scenes
		var currentScene by AppState . currentScene

		for ( device in Config . devices ) AppState . putDevice (device)
		for ( scene in Config . scenes ) AppState . putScene (scene)

		enableEdgeToEdge ()
		setContent {
			ZanburTheme {
				Scaffold { scaffoldPadding ->
					val wrapperModifier = Modifier
						.padding(scaffoldPadding)
						.fillMaxWidth()
						.fillMaxHeight()

					Column ( wrapperModifier , Center , CenterHorizontally ) {
						SceneSwitcher ( scenes . values . map { it . name } , currentScene ?. name ) {
							val newScene = scenes . getValue (it)
							currentScene = newScene
							handleSceneChange (newScene)
						}
					}
				}
			}
		}

		MqttClient . run (lifecycleScope)
	}
}

@Composable private fun SceneSwitcher (
	scenes : Collection <SceneName> ,
	currentScene : SceneName ? ,
	onSwitch : (SceneName) -> Unit ,
) = Column {
	for ( scene in scenes ) {
		val colors =
			if ( scene == currentScene ) ButtonDefaults . buttonColors ()
			else ButtonDefaults . filledTonalButtonColors ()

		Button ( { onSwitch (scene) } , Modifier , true , ButtonDefaults . shape , colors ) {
			Text ( scene . value )
		}
	}
}

private fun handleSceneChange ( newScene : Scene ) {
	for ( ( device , newState ) in newScene . states ) {
		val jsonString = when (device) {
			is TemperatureLamp -> Json . encodeToString ( newState as TemperatureLampState )
			is RgbLamp -> Json . encodeToString ( newState as RgbLampState )
			else -> throw Error ("Unknown type of device state")
		}
		MqttClient . publish (
			Config . MQTT_TOPIC + "/" + device . name . value + "/set" ,
			jsonString . toByteArray () . asUByteArray () ,
		)
	}
}

//private fun handlePowerChange ( deviceName : String , power : Power ) {
//	when ( val device = AppState . devices . value . getValue (deviceName) ) {
//
//		is TemperatureLamp ->
//			if ( device . currentState != null && device . currentState . power != power ) {
//				MqttClient . publish (
//					"zigbee2mqtt/$deviceName/set/state" ,
//					power . toString () . toByteArray () . asUByteArray () ,
//				)
//				AppState . putDevice ( TemperatureLamp ( deviceName , device . currentState . copy (power) ) )
//			}
//
//		is RgbLamp ->
//			if ( device . currentState != null && device . currentState . power != power ) {
//				MqttClient . publish (
//					"zigbee2mqtt/$deviceName/set/state" ,
//					power . toString () . toByteArray () . asUByteArray () ,
//				)
//				AppState . putDevice ( RgbLamp ( deviceName , device . currentState . copy (power) ) )
//			}
//	}
//}
