package com.kernelmaft.zanbur

import kotlinx.serialization.SerialName
import kotlinx.serialization.Serializable


typealias Real = Double


data class Scene (
	val name : SceneName ,
	val states : Map < Device , LampState > ,
)

data class SceneName ( val value : String )


abstract class Device ( val name : DeviceName )

class TemperatureLamp ( name : DeviceName ) : Device (name)
class RgbLamp ( name : DeviceName ) : Device (name)

data class DeviceName ( val value : String )


abstract class LampState {
	abstract val power : Power
	abstract val brightness : Int  // Range 0 to 254
}

@Serializable data class TemperatureLampState (
	@SerialName ("state") override val power : Power ,
	@SerialName ("brightness") override val brightness : Int ,
	@SerialName ("color_temp") val colorTemperature : Int ,  // Range 250 to 454
) : LampState ()

@Serializable data class RgbLampState (
	@SerialName ("state") override val power : Power ,
	@SerialName ("brightness") override val brightness : Int ,
	@SerialName ("color") val color : Cie1931Color ,
) : LampState ()

@Serializable enum class Power {
	ON , OFF ;

	fun toBoolean () : Boolean = when (this) {
		ON -> true
		OFF -> false
	}

	companion object {
		fun fromBoolean ( power : Boolean ) : Power = when (power) {
			true -> ON
			false -> OFF
		}
	}
}

@Serializable data class Cie1931Color (
	val x : Real ,  // Range 0.0 to 1.0 (for shrimp anyway)
	val y : Real ,  // Idem
)
